import pytest
from pytest import fixture
from fractions import Fraction
from poisson_approval import ProfileTwelve, StrategyTwelve, StrategyOrdinal, EquilibriumStatus, \
    APPROVAL, PLURALITY, ANTI_PLURALITY, TauVector, initialize_random_seeds, UTILITY_DEPENDENT, SPLIT


def test_iterative_voting_verbose():
    """
    >>> from fractions import Fraction
    >>> profile = ProfileTwelve(d_type_share={'a_bc': Fraction(1, 2), 'ab_c': Fraction(1, 2)})
    >>> strategy = StrategyTwelve(d_ranking_ballot={'abc': 'ab'})
    >>> result = profile.iterated_voting(init=strategy, n_max_episodes=100,
    ...                                  ballot_update_ratio=1, verbose=True)
    t = 0
    strategy: <abc: ab> ==> a, b
    tau_actual: <ab: 1> ==> a, b
    t = 1
    tau_perceived: <ab: 1> ==> a, b
    mu_ab > mu_ac = mu_bc
    strategy: <abc: a> ==> a
    tau_full_response: <a: 1> ==> a
    tau_actual: <a: 1> ==> a
    t = 2
    tau_perceived: <a: 1> ==> a
    mu_ab = mu_ac = mu_bc
    strategy: <abc: a> ==> a
    tau_full_response: <a: 1> ==> a
    tau_actual: <a: 1> ==> a
    t = 3
    tau_perceived: <a: 1> ==> a
    mu_ab = mu_ac = mu_bc
    strategy: <abc: a> ==> a
    tau_full_response: <a: 1> ==> a
    tau_actual: <a: 1> ==> a
    >>> result = profile.fictitious_play(init=strategy, n_max_episodes=100,
    ...                                  perception_update_ratio=1, ballot_update_ratio=1, verbose=True)
    t = 0
    strategy: <abc: ab> ==> a, b
    tau_actual: <ab: 1> ==> a, b
    t = 1
    tau_perceived: <ab: 1> ==> a, b
    mu_ab > mu_ac = mu_bc
    strategy: <abc: a> ==> a
    tau_full_response: <a: 1> ==> a
    tau_actual: <a: 1> ==> a
    t = 2
    tau_perceived: <a: 1> ==> a
    mu_ab = mu_ac = mu_bc
    strategy: <abc: a> ==> a
    tau_full_response: <a: 1> ==> a
    tau_actual: <a: 1> ==> a
    """
    pass


@fixture()
def my_profile():
    return ProfileTwelve(d_type_share={'a_bc': 1, 'ab_c': 1})


@fixture()
def my_strategy():
    return StrategyTwelve(d_ranking_ballot={'abc': 'ab'})


def test_normalization(my_profile):
    assert my_profile.a_bc == 0.5


def test_not_equilibrium(my_profile, my_strategy):
    assert my_profile.is_equilibrium(my_strategy) == EquilibriumStatus.NOT_EQUILIBRIUM


def test_iterated_voting_with_cycle():
    """
        >>> my_profile = ProfileTwelve(d_type_share={'a_bc': 1, 'ab_c': 1}, voting_rule=ANTI_PLURALITY)
        >>> my_strategy = StrategyTwelve(d_ranking_ballot={'abc': 'ab'})
        >>> def share_double_votes_t(tau):
        ...     return tau.ab + tau.ac + tau.bc
        >>> def share_double_votes_s(strategy):
        ...     return strategy.share_double_votes
        >>> result = my_profile.iterated_voting(
        ...     init=my_strategy, n_max_episodes=10, ballot_update_ratio=1,
        ...     other_statistics_tau={'share_double_votes_t': share_double_votes_t},
        ...     other_statistics_strategy={'share_double_votes_s': share_double_votes_s})
        >>> result['cycle_taus_actual']
        [TauVector({'ab': 1}, voting_rule='Anti-plurality'), TauVector({'ac': 1}, voting_rule='Anti-plurality')]
        >>> result['d_candidate_winning_frequency']
        {'a': Fraction(1, 2), 'b': Fraction(1, 4), 'c': Fraction(1, 4)}
        >>> result['share_double_votes_t']
        1.0
        >>> result['share_double_votes_s']
        1.0
    """
    pass


def test_iterated_voting_without_convergence():
    """
        >>> my_profile = ProfileTwelve(
        ...     d_type_share={'ab_c': Fraction(2, 5), 'c_ba': Fraction(2, 5), 'ca_b': Fraction(1, 5)})
        >>> my_strategy = StrategyTwelve(d_ranking_ballot={'abc': 'a', 'cab': 'ac', 'cba': 'bc'})
        >>> def share_double_votes_t(tau):
        ...     return tau.ab + tau.ac + tau.bc
        >>> def share_double_votes_s(strategy):
        ...     return strategy.share_double_votes
        >>> result = my_profile.iterated_voting(
        ...     init=my_strategy, n_max_episodes=4, ballot_update_ratio=1,
        ...     other_statistics_tau={'share_double_votes_t': share_double_votes_t},
        ...     other_statistics_strategy={'share_double_votes_s': share_double_votes_s})
        >>> result['cycle_taus_actual']
        []
        >>> result['d_candidate_winning_frequency']
        {'a': Fraction(1, 8), 'c': Fraction(7, 8)}
        >>> result['share_double_votes_t']  # doctest: +ELLIPSIS
        0.4499999...
        >>> result['share_double_votes_s']
        Fraction(9, 20)
    """
    pass


def test_fictitious_play_without_convergence():
    """
        >>> my_profile = ProfileTwelve(d_type_share={'a_bc': 1, 'ab_c': 1}, voting_rule=ANTI_PLURALITY)
        >>> my_strategy = StrategyTwelve(d_ranking_ballot={'abc': 'ab'})
        >>> def share_double_votes_t(tau):
        ...     return tau.ab + tau.ac + tau.bc
        >>> def share_double_votes_s(strategy):
        ...     return strategy.share_double_votes
        >>> result = my_profile.fictitious_play(
        ...     init=my_strategy, n_max_episodes=10,
        ...     other_statistics_tau={'share_double_votes_t': share_double_votes_t},
        ...     other_statistics_strategy={'share_double_votes_s': share_double_votes_s})
        >>> print(result['tau'])
        None
        >>> result['d_candidate_winning_frequency']
        {'a': Fraction(1, 2), 'b': Fraction(1, 4), 'c': Fraction(1, 4)}
        >>> result['share_double_votes_t']
        Fraction(1, 1)
        >>> result['share_double_votes_s']
        Fraction(1, 1)
    """
    pass


def test_plurality():
    """
        >>> profile = ProfileTwelve(
        ...     d_type_share={'a_bc': Fraction(1, 5), 'ba_c': Fraction(3, 5), 'c_ab': Fraction(1, 5)},
        ...     voting_rule=PLURALITY)
        >>> profile
        ProfileTwelve({'a_bc': Fraction(1, 5), 'ba_c': Fraction(3, 5), 'c_ab': Fraction(1, 5)}, voting_rule='Plurality')
        >>> print(profile)
        <a_bc: 1/5, ba_c: 3/5, c_ab: 1/5> (Condorcet winner: b) (Plurality)
        >>> profile.tau_sincere
        TauVector({'a': Fraction(1, 5), 'b': Fraction(3, 5), 'c': Fraction(1, 5)}, voting_rule='Plurality')
        >>> profile.tau_fanatic
        TauVector({'a': Fraction(1, 5), 'b': Fraction(3, 5), 'c': Fraction(1, 5)}, voting_rule='Plurality')
        >>> strategy = StrategyTwelve({'abc': 'a', 'bac': 'b', 'cab': 'c'}, profile=profile)
        >>> strategy.is_equilibrium
        EquilibriumStatus.EQUILIBRIUM
    """
    pass


def test_anti_plurality():
    """
        >>> profile = ProfileTwelve(
        ...     d_type_share={'ab_c': Fraction(2, 5), 'b_ca': Fraction(2, 5), 'ca_b': Fraction(1, 5)},
        ...     voting_rule=ANTI_PLURALITY)
        >>> profile
        ProfileTwelve({'ab_c': Fraction(2, 5), 'b_ca': Fraction(2, 5), 'ca_b': Fraction(1, 5)}, \
voting_rule='Anti-plurality')
        >>> print(profile)
        <ab_c: 2/5, b_ca: 2/5, ca_b: 1/5> (Anti-plurality)
        >>> profile.tau_sincere
        TauVector({'ab': Fraction(2, 5), 'ac': Fraction(1, 5), 'bc': Fraction(2, 5)}, voting_rule='Anti-plurality')
        >>> profile.tau_fanatic
        TauVector({'ab': Fraction(2, 5), 'ac': Fraction(1, 5), 'bc': Fraction(2, 5)}, voting_rule='Anti-plurality')
        >>> strategy = StrategyTwelve({'abc': 'ab', 'bca': 'bc', 'cab': 'ac'}, profile=profile)
        >>> strategy.is_equilibrium
        EquilibriumStatus.EQUILIBRIUM
    """
    pass


def test_initializer():
    """
        >>> profile = ProfileTwelve(
        ...     d_type_share={'ab_c': Fraction(2, 5), 'b_ca': Fraction(2, 5), 'ca_b': Fraction(1, 5)})
        >>> profile._initializer(init=StrategyTwelve({'abc': 'ab', 'bca': 'bc', 'cab': 'ac'}))
        (StrategyTwelve({'abc': 'ab', 'bca': 'bc', 'cab': 'ac'}), \
TauVector({'ab': Fraction(2, 5), 'ac': Fraction(1, 5), 'bc': Fraction(2, 5)}))
        >>> profile._initializer(TauVector({'ab': Fraction(2, 5), 'ac': Fraction(1, 5), 'bc': Fraction(2, 5)}))
        (None, TauVector({'ab': Fraction(2, 5), 'ac': Fraction(1, 5), 'bc': Fraction(2, 5)}))
        >>> profile._initializer('sincere')
        (None, TauVector({'ab': Fraction(2, 5), 'ac': Fraction(1, 5), 'b': Fraction(2, 5)}))
        >>> profile._initializer('fanatic')
        (None, TauVector({'a': Fraction(2, 5), 'b': Fraction(2, 5), 'c': Fraction(1, 5)}))
        >>> initialize_random_seeds()
        >>> profile._initializer('random_tau')
        (None, TauVector({'a': 0.4236547993389047, 'ab': 0.12122838365799216, 'ac': 0.0039303209304278885, \
'b': 0.05394987214431912, 'bc': 0.1124259903007756, 'c': 0.2848106336275805}))
        >>> initialize_random_seeds()
        >>> profile._initializer('random_tau_undominated')
        (None, TauVector({'a': 0.33776874061001927, 'ab': 0.06223125938998075, 'ac': 0.0977450557262783, \
'b': 0.10356670011718534, 'bc': 0.2964332998828147, 'c': 0.10225494427372171}))

        >>> profile = ProfileTwelve(d_type_share={'ab_c': 1})
        >>> initialize_random_seeds()
        >>> profile._initializer('random_tau_undominated')
        (None, TauVector({'a': 0.8444218515250481, 'ab': 0.15557814847495188}))
    """
    with pytest.warns(Warning):
        ProfileTwelve({'a_bc': 1}, symbolic=True)._initializer(init='random_tau')
    with pytest.raises(ValueError):
        ProfileTwelve({'a_bc': 1})._initializer(init='unexpected argument')


def test_random_tau_undominated_with_weak_orders():
    """
        >>> initialize_random_seeds()
        >>> profile = ProfileTwelve({'a>b~c': Fraction(9, 10), 'b~c>a': Fraction(1, 10)}, voting_rule=APPROVAL)
        >>> profile.random_tau_undominated()
        TauVector({'a': 0.9, 'bc': 0.1})
        >>> profile = ProfileTwelve({'a>b~c': Fraction(9, 10), 'b~c>a': Fraction(1, 10)}, voting_rule=PLURALITY)
        >>> profile.random_tau_undominated()
        TauVector({'a': 0.9, 'b': 0.06183689966753317, 'c': 0.03816310033246684}, voting_rule='Plurality')
        >>> profile = ProfileTwelve({'a>b~c': Fraction(9, 10), 'b~c>a': Fraction(1, 10)}, voting_rule=ANTI_PLURALITY)
        >>> profile.random_tau_undominated()
        TauVector({'ab': 0.6568485734341158, 'ac': 0.24315142656588423, 'bc': 0.1}, voting_rule='Anti-plurality')
    """
    pass


def test_strategy_weak_order():
    """
        >>> profile = ProfileTwelve({'ab_c': Fraction(1, 7), 'a~b>c': Fraction(2, 7), 'c>a~b': Fraction(4, 7)})
        >>> strategy = StrategyOrdinal({'abc': 'a'}, profile=profile)
        >>> print(strategy.tau)
        <a: 1/7, ab: 2/7, c: 4/7> ==> c
    """
    pass


def test_welfare():
    """
        >>> profile = ProfileTwelve({'a~b>c': Fraction(1, 3), 'a>b~c': Fraction(2, 3)})
        >>> profile.d_candidate_welfare
        {'a': Fraction(1, 1), 'b': Fraction(1, 3), 'c': 0}
    """
    pass


def test_share_sincere_among_strategic_voters_approval():
    """
        >>> profile = ProfileTwelve({'a_bc': Fraction(1, 3), 'ab_c': Fraction(2, 3)})
        >>> strategy = StrategyTwelve({'abc': 'a'}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(1, 3)
        >>> strategy = StrategyTwelve({'abc': UTILITY_DEPENDENT}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        1
        >>> strategy = StrategyTwelve({'abc': 'ab'}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(2, 3)
    """
    pass


def test_share_sincere_among_strategic_voters_plurality():
    """
        >>> profile = ProfileTwelve({'a_bc': Fraction(1, 3), 'ab_c': Fraction(2, 3)}, voting_rule=PLURALITY)
        >>> strategy = StrategyTwelve({'abc': 'a'}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        1
        >>> strategy = StrategyTwelve({'abc': UTILITY_DEPENDENT}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(1, 3)
        >>> strategy = StrategyTwelve({'abc': 'b'}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        0
    """
    pass


def test_share_sincere_among_strategic_voters_anti_plurality():
    """
        >>> profile = ProfileTwelve({'a_bc': Fraction(1, 3), 'ab_c': Fraction(2, 3)}, voting_rule=ANTI_PLURALITY)
        >>> strategy = StrategyTwelve({'abc': 'ab'}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        1
        >>> strategy = StrategyTwelve({'abc': UTILITY_DEPENDENT}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(2, 3)
        >>> strategy = StrategyTwelve({'abc': 'ac'}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        0
    """
    pass


def test_d_ballot_share_weak_voters_strategic():
    """
        >>> profile = ProfileTwelve({}, d_weak_order_share={'a~b>c': 0.5, 'a~c>b': 0.3, 'b>a~c': 0.2},
        ...                                                 voting_rule=PLURALITY)
        >>> strategy = StrategyTwelve({}, d_weak_order_ballot={'a~b>c': 'a', 'a~c>b': SPLIT}, profile=profile)
        >>> profile.d_ballot_share_weak_voters_strategic(strategy)
        {'a': 0.65, 'b': 0.2, 'c': 0.15, 'ab': 0, 'ac': 0, 'bc': 0}

        >>> profile = ProfileTwelve({}, d_weak_order_share={'a>b~c': 0.5, 'b>a~c': 0.3, 'a~c>b': 0.2},
        ...                                                 voting_rule=ANTI_PLURALITY)
        >>> strategy = StrategyTwelve({}, d_weak_order_ballot={'a>b~c': 'ab', 'b>a~c': SPLIT}, profile=profile)
        >>> profile.d_ballot_share_weak_voters_strategic(strategy)
        {'a': 0, 'b': 0, 'c': 0, 'ab': 0.65, 'ac': 0.2, 'bc': 0.15}
    """
    pass


def test_best_responses_to_strategy():
    """
        >>> profile = ProfileTwelve({}, d_weak_order_share={'a~b>c': 0.5, 'a~c>b': 0.3, 'b~c>a': 0.2},
        ...                                                 voting_rule=PLURALITY)
        >>> tau = TauVector({'a': 0.4, 'b': 0.2, 'c': 0.4}, voting_rule=PLURALITY)
        >>> profile.best_responses_to_strategy(tau)
        StrategyThreshold({}, d_weak_order_ballot={'a~b>c': 'a', 'a~c>b': 'Split', 'b~c>a': 'c'}, voting_rule='Plurality')

        >>> profile = ProfileTwelve({}, d_weak_order_share={'a>b~c': 0.5, 'b>a~c': 0.3, 'c>a~b': 0.2},
        ...                                                 voting_rule=ANTI_PLURALITY)
        >>> tau = TauVector({'ab': 0.2, 'ac': 0.6, 'bc': 0.2}, voting_rule=ANTI_PLURALITY)
        >>> profile.best_responses_to_strategy(tau)
        StrategyThreshold({}, d_weak_order_ballot={'a>b~c': 'ab', 'b>a~c': 'Split', 'c>a~b': 'bc'}, voting_rule='Anti-plurality')
    """
    pass



from poisson_approval import binary_figure


def test_heatmap_intensity_reverse():
    """
        >>> def f(x, y1, y2):
        ...     return (x**2 + y1) / (y2 + 1)
        >>> figure, tax = binary_figure(xscale=5, yscale=5, size_inches=(6, 4))
        >>> tax.heatmap_intensity(f,
        ...                       x_left_label='x-left',
        ...                       x_right_label='x-right',
        ...                       y_left_label='y-left',
        ...                       y_right_label='y-right',
        ...                       reverse_right=True)
        >>> tax.set_title('An intensity heat map')
    """
    pass


def test_heatmap_candidates_reverse():
    """
        >>> def g(x, y1, y2):
        ...     a = x**.5
        ...     b = y1**2
        ...     c = 1 - a - b
        ...     return [a, b, c]
        >>> figure, tax = binary_figure(xscale=5, yscale=5, size_inches=(6, 4))
        >>> tax.heatmap_candidates(g,
        ...                        x_left_label='x-left',
        ...                        x_right_label='x-right',
        ...                        y_left_label='y-left',
        ...                        y_right_label='y-right',
        ...                        legend_title='Candidates',
        ...                        legend_style='palette',
        ...                        reverse_right=True)
        >>> tax.set_title('A candidate heat map')
    """
    pass



from fractions import Fraction
from poisson_approval import ProfileDiscrete, StrategyOrdinal, PLURALITY, ANTI_PLURALITY, StrategyThreshold


def test_normalization():
    profile = ProfileDiscrete({('abc', 0.2): 1, ('acb', 0.4): 1})
    assert profile.abc == 0.5


def test_plurality():
    """
        >>> profile = ProfileDiscrete({('abc', 0.2): 1, ('acb', 0.4): 1}, voting_rule=PLURALITY)
        >>> profile
        ProfileDiscrete({'abc': {0.2: Fraction(1, 2)}, 'acb': {0.4: Fraction(1, 2)}}, voting_rule='Plurality')
        >>> print(profile)
        <abc 0.2: 1/2, acb 0.4: 1/2> (Condorcet winner: a) (Plurality)
    """
    pass


def test_anti_plurality():
    """
        >>> profile = ProfileDiscrete({('abc', 0.2): 1, ('acb', 0.4): 1}, voting_rule=ANTI_PLURALITY)
        >>> profile
        ProfileDiscrete({'abc': {0.2: Fraction(1, 2)}, 'acb': {0.4: Fraction(1, 2)}}, voting_rule='Anti-plurality')
        >>> print(profile)
        <abc 0.2: 1/2, acb 0.4: 1/2> (Condorcet winner: a) (Anti-plurality)
    """
    pass


def test_misc_forms_of_input():
    """
        >>> profile = ProfileDiscrete({('abc', 0.2): 0, ('abc', 0): Fraction(3, 7), ('bca', 1): Fraction(4, 7)})
        >>> profile
        ProfileDiscrete({}, d_weak_order_share={'a>b~c': Fraction(3, 7), 'b~c>a': Fraction(4, 7)})
        >>> print(profile)
        <a>b~c: 3/7, b~c>a: 4/7> (Condorcet winner: b, c)
    """
    pass


def test_strategy_weak_order():
    """
        >>> profile = ProfileDiscrete({('abc', 0.3): Fraction(1, 7), 'a~b>c': Fraction(2, 7), 'c>a~b': Fraction(4, 7)})
        >>> strategy = StrategyOrdinal({'abc': 'a'}, profile=profile)
        >>> print(strategy.tau)
        <a: 1/7, ab: 2/7, c: 4/7> ==> c
    """
    pass


def test_mixed_strategy():
    """
        >>> profile = ProfileDiscrete({('abc', 0.2): Fraction(1, 7), ('acb', 0.4): Fraction(6, 7)})
        >>> strategy = StrategyThreshold({'abc': (0.2, Fraction(1, 3)), 'acb': 0}, profile=profile)
        >>> print(strategy.tau)
        <a: 1/21, ab: 2/21, ac: 6/7> ==> a
    """
    pass


def test_welfare():
    """
        >>> profile = ProfileDiscrete({
        ...     ('abc', Fraction(3, 10)): Fraction(26, 100), ('bac', Fraction(1, 10)): Fraction(21, 100),
        ...     'a~b>c': Fraction(23, 100), 'a>b~c': Fraction(30, 100)
        ... })
        >>> profile.d_candidate_welfare
        {'a': Fraction(811, 1000), 'b': Fraction(259, 500), 'c': 0}
    """
    pass


def test_relative_welfare():
    """
        >>> profile = ProfileDiscrete({
        ...     ('abc', Fraction(1, 2)): Fraction(1, 3),
        ...     ('bca', Fraction(1, 2)): Fraction(1, 3),
        ...     ('cab', Fraction(1, 2)): Fraction(1, 3)
        ... })
        >>> profile.d_candidate_relative_welfare
        {'a': 1, 'b': 1, 'c': 1}
    """
    pass


def test_share_sincere_among_strategic_voters_weak_orders():
    """
        >>> profile = ProfileDiscrete({'a~b>c': 1})
        >>> strategy = StrategyOrdinal({})
        >>> profile.share_sincere_among_strategic_voters(strategy)
        1
    """
    pass


def test_share_sincere_among_strategic_voters_approval():
    """
        >>> profile = ProfileDiscrete({
        ...     ('abc', Fraction(2, 11)):   Fraction(1, 7),   # Sincere vote: a
        ...     ('abc', Fraction(1, 2)):    Fraction(2, 7),   # Sincere vote: a
        ...     ('abc', Fraction(9, 11)):   Fraction(4, 7),   # Sincere vote: ab
        ... })
        >>> strategy = StrategyThreshold({'abc': 0})
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(4, 7)
        >>> strategy = StrategyThreshold({'abc': Fraction(2, 11)}, ratio_optimistic=Fraction(1, 3))
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(13, 21)
        >>> strategy = StrategyThreshold({'abc': Fraction(1, 2)}, ratio_optimistic=Fraction(1, 3))
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(17, 21)
        >>> strategy = StrategyThreshold({'abc': Fraction(9, 11)}, ratio_optimistic=Fraction(1, 3))
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(17, 21)
        >>> strategy = StrategyThreshold({'abc': 1})
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(3, 7)
    """
    pass


def test_share_sincere_among_strategic_voters_plurality():
    """
        >>> profile = ProfileDiscrete({
        ...     ('abc', Fraction(2, 11)):   Fraction(1, 7),
        ...     ('abc', Fraction(1, 2)):    Fraction(2, 7),
        ...     ('abc', Fraction(9, 11)):   Fraction(4, 7),
        ... }, voting_rule=PLURALITY)
        >>> strategy = StrategyThreshold({'abc': 0}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        0
        >>> strategy = StrategyThreshold({'abc': Fraction(2, 11)}, ratio_optimistic=Fraction(1, 3), profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(1, 21)
        >>> strategy = StrategyThreshold({'abc': Fraction(1, 2)}, ratio_optimistic=Fraction(1, 3), profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(5, 21)
        >>> strategy = StrategyThreshold({'abc': Fraction(9, 11)}, ratio_optimistic=Fraction(1, 3), profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(13, 21)
        >>> strategy = StrategyThreshold({'abc': 1}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        1
    """
    pass


def test_share_sincere_among_strategic_voters_anti_plurality():
    """
        >>> profile = ProfileDiscrete({
        ...     ('abc', Fraction(2, 11)):   Fraction(1, 7),
        ...     ('abc', Fraction(1, 2)):    Fraction(2, 7),
        ...     ('abc', Fraction(9, 11)):   Fraction(4, 7),
        ... }, voting_rule=ANTI_PLURALITY)
        >>> strategy = StrategyThreshold({'abc': 0}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        1
        >>> strategy = StrategyThreshold({'abc': Fraction(2, 11)}, ratio_optimistic=Fraction(1, 3), profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(20, 21)
        >>> strategy = StrategyThreshold({'abc': Fraction(1, 2)}, ratio_optimistic=Fraction(1, 3), profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(16, 21)
        >>> strategy = StrategyThreshold({'abc': Fraction(9, 11)}, ratio_optimistic=Fraction(1, 3), profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        Fraction(8, 21)
        >>> strategy = StrategyThreshold({'abc': 1}, profile=profile)
        >>> profile.share_sincere_among_strategic_voters(strategy)
        0
    """
    pass


def test_share_sincere_among_fanatic_voters_weak_orders():
    """
        >>> profile = ProfileDiscrete({'a~b>c': 1})
        >>> profile.share_sincere_among_fanatic_voters
        1
    """
    pass


def test_share_sincere_among_fanatic_voters_approval():
    """
        >>> profile = ProfileDiscrete({
        ...     ('abc', Fraction(2, 11)):   Fraction(1, 7),   # Sincere vote: a
        ...     ('abc', Fraction(1, 2)):    Fraction(2, 7),   # Sincere vote: a
        ...     ('abc', Fraction(9, 11)):   Fraction(4, 7),   # Sincere vote: ab
        ... })
        >>> profile.share_sincere_among_fanatic_voters
        Fraction(3, 7)
    """
    pass


def test_share_sincere_among_fanatic_voters_plurality():
    """
        >>> profile = ProfileDiscrete({
        ...     ('abc', Fraction(2, 11)):   Fraction(1, 7),
        ...     ('abc', Fraction(1, 2)):    Fraction(2, 7),
        ...     ('abc', Fraction(9, 11)):   Fraction(4, 7),
        ... }, voting_rule=PLURALITY)
        >>> profile.share_sincere_among_fanatic_voters
        1
    """
    pass


def test_share_sincere_among_fanatic_voters_anti_plurality():
    """
        >>> profile = ProfileDiscrete({
        ...     ('abc', Fraction(2, 11)):   Fraction(1, 7),
        ...     ('abc', Fraction(1, 2)):    Fraction(2, 7),
        ...     ('abc', Fraction(9, 11)):   Fraction(4, 7),
        ... }, voting_rule=ANTI_PLURALITY)
        >>> profile.share_sincere_among_fanatic_voters
        1
    """
    pass


def test_share_sincere():
    """
        >>> profile = ProfileDiscrete({
        ...     ('abc', Fraction(2, 11)): 1,   # Sincere vote: a
        ... }, ratio_fanatic=Fraction(1, 3))
        >>> strategy = StrategyOrdinal({'abc': 'ab'})
        >>> profile.share_sincere_among_fanatic_voters
        1
        >>> profile.share_sincere_among_strategic_voters(strategy)
        0
        >>> profile.share_sincere(strategy)
        Fraction(1, 3)

        >>> profile = ProfileDiscrete({
        ...     ('abc', Fraction(2, 11)): 1,   # Sincere vote: a
        ... }, ratio_sincere=Fraction(1, 5))
        >>> strategy = StrategyOrdinal({'abc': 'ab'})
        >>> profile.share_sincere_among_strategic_voters(strategy)
        0
        >>> profile.share_sincere(strategy)
        Fraction(1, 5)
    """
    pass



from poisson_approval.constants.basic_constants import *
from poisson_approval.random_factories.RandSimplexUniform import RandSimplexUniform
from poisson_approval.utils.Util import initialize_random_seeds
from poisson_approval.profiles.ProfileNoisyDiscrete import ProfileNoisyDiscrete


class RandProfileNoisyDiscreteUniform(RandSimplexUniform):
    """A random factory of noisy discrete profiles (:class:`ProfileNoisyDiscrete`) following the uniform distribution.

    Parameters
    ----------
    types : iterable
        These types will have a variable share. They can be noisy discrete types, e.g. ``('abc', 0.9, 0.01)``,
        or discrete types, e.g. ``('abc', 0.9)`` (in which case the argument `noise` must be given in the additional
        parameters), or weak orders, e.g. ``'a~b>c'``.
    d_type_fixed_share : dict, optional
        A dictionary. For each entry ``type: fixed_share``, this type will have at least this fixed share. The total
        must be lower or equal to 1.
    kwargs
        These additional arguments will be passed directly to :class:`ProfileNoisyDiscrete`.

    Examples
    --------
    Basic usage:

        >>> initialize_random_seeds()
        >>> rand_profile = RandProfileNoisyDiscreteUniform(types=[('abc', 0.9, 0.01), 'a~b>c'])
        >>> profile = rand_profile()
        >>> print(profile)
        <abc 0.9 ± 0.01: 0.5488135039273248, a~b>c: 0.45118649607267525> (Condorcet winner: a)

    Or, equivalently:

        >>> initialize_random_seeds()
        >>> rand_profile = RandProfileNoisyDiscreteUniform(types=[('abc', 0.9), 'a~b>c'], noise=0.01)
        >>> profile = rand_profile()
        >>> print(profile)
        <abc 0.9 ± 0.01: 0.5488135039273248, a~b>c: 0.45118649607267525> (Condorcet winner: a)

    Using the optional parameters:

        >>> rand_profile = RandProfileNoisyDiscreteUniform(
        ...     types=[('abc', 0.9, 0.01), 'a~b>c'], d_type_fixed_share={'b>a~c': 0.5},
        ...     voting_rule=PLURALITY)
        >>> profile = rand_profile()
        >>> print(profile)
        <abc 0.9 ± 0.01: 0.35759468318620974, a~b>c: 0.14240531681379026, b>a~c: 0.5> (Condorcet winner: b) (Plurality)

    For more examples, cf. :class:`RandSimplexUniform`.
    """

    def __init__(self, types, d_type_fixed_share=None, **kwargs):
        super().__init__(cls=ProfileNoisyDiscrete, keys=types, d_key_fixed_share=d_type_fixed_share, **kwargs)



from poisson_approval.constants.basic_constants import *
from poisson_approval.iterables.IterableSimplexGrid import IterableSimplexGrid
from poisson_approval.profiles.ProfileHistogram import ProfileHistogram


class IterableProfileHistogramGrid:
    """Iterate over histogram profile (:class:`ProfileHistogram`) defined on a grid.

    Parameters
    ----------
    denominator : int or iterable
        The grain(s) of the grid.
    types : iterable
        These types will have a variable share. They must eih=ther give a ranking and a histogram, e.g.
        ``('abc', (0.2, 0.1, 0.4, 0.3))``, or be a weak order, e.g. ``'a~b>c'``.
    d_type_fixed_share : dict, optional
        A dictionary. For each entry ``type: fixed_share``, this type will have at least this fixed share. The total
        must be lower or equal to 1.
    standardized : bool, optional
        If True, then only standardized profiles are given. Cf. :meth:`Profile.is_standardized`. You should
        probably use this option if the arguments `types`, `d_type_fixed_share` and `test` treat the candidates
        symmetrically.
    test : callable, optional
        A function ``ProfileHistogram -> bool``. Only profiles meeting this test are given.
    kwargs
        Additional parameters are passed to :class:`ProfileHistogram` when creating the profile.

    Examples
    --------
    Basic usage:

        >>> for profile in IterableProfileHistogramGrid(denominator=3, types=[('abc', (0.3, 0.7)), 'a~b>c']):
        ...     print(profile)
        <abc: 1 [0.3 0.7]> (Condorcet winner: a)
        <abc: 2/3 [0.3 0.7], a~b>c: 1/3> (Condorcet winner: a)
        <abc: 1/3 [0.3 0.7], a~b>c: 2/3> (Condorcet winner: a)
        <a~b>c: 1> (Condorcet winner: a, b)

    For more examples, cf. :class:`IterableSimplexGrid`.
    """
    def __init__(self, denominator, types, d_type_fixed_share=None, standardized=False, test=None, **kwargs):
        self.standardized = standardized
        self._base_iterator = IterableSimplexGrid(cls=ProfileHistogram, denominator=denominator, keys=types,
                                                  d_key_fixed_share=d_type_fixed_share, test=test, **kwargs)

    def __iter__(self):
        return (profile for profile in self._base_iterator
                if not self.standardized or profile.is_standardized)



from fractions import Fraction
import numpy as np
from poisson_approval.meta_analysis.binary_plots import binary_figure
from poisson_approval.profiles.ProfileNoisyDiscrete import ProfileNoisyDiscrete
from poisson_approval.utils.Util import candidates_to_probabilities, one_over_log_t_plus_one, d_candidate_value_to_array


class XyyToProfile:
    """Map a point `(x, y1, y2)` to a profile (for binary plots).

    Parameters
    ----------
    cls : class
        :class:`ProfileDiscrete` or :class:`ProfileNoisyDiscrete`.
    left_ranking,right_ranking : str
        A ranking, whose share is maximal on the left (resp. on the right).
    d_type_fixed_share : dict
        Key: a type that is suitable for the kind of profile used. Value: a share of voters.
    kwargs
        Other keyword arguments are passed to the profile. E.g. ``voting_rule``, ``ratio_fanatic``, etc.

    Notes
    -----
    An `XyyToProfile` object is a callable. When called, it takes three parameters `x`, `y1`, `y2` in the interval
    [0, 1]. It outputs the profile defined by:

    * The class is given by `cls`,
    * According to `d_type_fixed_share`, some types are assigned fixed shares of voters.
    * The other voters are distributed between ``left_ranking`` and ``right_ranking``, in respective proportions that
      are given by `1 - x` and `x`.
    * The voters of `left_ranking` have a utility `y1` for their second candidate.
    * The voters of `right_ranking` have a utility `y2` for their second candidate.

    Examples
    --------
    Typical usage:

        >>> xyy_to_profile = XyyToProfile(
        ...     ProfileNoisyDiscrete,
        ...     d_type_fixed_share={('abc', 0.4, 0.01): Fraction(2, 11)},
        ...     left_ranking='bca', right_ranking='cab',
        ...     noise=0.01)
        >>> profile = xyy_to_profile(x=Fraction(4, 9), y1=0.7, y2=0.9)
        >>> print(profile)
        <abc 0.4 ± 0.01: 2/11, bca 0.7 ± 0.01: 5/11, cab 0.9 ± 0.01: 4/11>

    The types with variable share and fixed share may overlap:

        >>> xyy_to_profile = XyyToProfile(
        ...     ProfileNoisyDiscrete,
        ...     d_type_fixed_share={('abc', 0.4, 0.01): Fraction(2, 11)},
        ...     left_ranking='abc', right_ranking='cab',
        ...     noise=0.01)
        >>> profile = xyy_to_profile(x=Fraction(4, 9), y1=0.4, y2=0.9)
        >>> print(profile)
        <abc 0.4 ± 0.01: 7/11, cab 0.9 ± 0.01: 4/11> (Condorcet winner: a)
    """

    def __init__(self, cls, left_ranking, right_ranking, d_type_fixed_share=None, **kwargs):
        if d_type_fixed_share is None:
            d_type_fixed_share = dict()
        self.cls = cls
        self.left_order, self.right_order = left_ranking, right_ranking
        self.d_type_fixed_share = d_type_fixed_share
        self.total_fixed_share = sum(self.d_type_fixed_share.values())
        self.variable_share = 1 - sum(self.d_type_fixed_share.values())
        self.kwargs = kwargs
        # Computed parameters
        self.d_order_fixed_share = dict()
        for t, share in d_type_fixed_share.items():
            order, _ = cls.order_and_label(t)
            self.d_order_fixed_share[order] = self.d_order_fixed_share.get(order, 0) + share
        self.x_left_label = '$r(%s)$' % self.left_order
        self.x_right_label = '$r(%s)$' % self.right_order
        self.y_left_label = '$u_%s$' % self.left_order[1]
        self.y_right_label = '$u_%s$' % self.right_order[1]

    def __call__(self, x, y1, y2):
        d_type_share = dict()
        d_type_share.update(self.d_type_fixed_share)
        d_type_share[(self.left_order, y1)] = d_type_share.get((self.left_order, y1), 0) + (1 - x) * self.variable_share
        d_type_share[(self.right_order, y2)] = d_type_share.get((self.right_order, y2), 0) + x * self.variable_share
        return self.cls(d_type_share, **self.kwargs)


def binary_plot_n_equilibria(xyy_to_profile, xscale, yscale, title='Number of equilibria',
                             meth='analyzed_strategies_ordinal', reverse_right=False, **kwargs):
    """Shortcut: binary plot for the number of equilibria.

    Parameters
    ----------
    xyy_to_profile : XyyToProfile
        This is responsible for generating the profiles.
    xscale : Number
        Scale of the plot (resolution) on the x-axis.
    yscale : Number
        Scale of the plot (resolution) on the y-axis.
    title : str
        Title of the plot.
    meth : str
        The name of the :class:`AnalyzedStrategies` property used to count the equilibria. Cf. :class:`Profile`.
    reverse_right : bool
        If True, then the y-axis on the right goes decreasing from 1 to 0 (whereas the y-axis on the left goes
        increasing from 0 to 1).
    kwargs
        Other keyword arguments are passed to the function :meth:`BinaryAxesSubplotPoisson.heatmap_intensity`.

    Examples
    --------
        >>> xyy_to_profile = XyyToProfile(ProfileNoisyDiscrete, left_ranking='bca', right_ranking='cab', noise=0.01)
        >>> figure, ax = binary_plot_n_equilibria(xyy_to_profile, xscale=5, yscale=5)
    """
    def n_equilibria(x, y1, y2):
        profile = xyy_to_profile(x, y1, y2)
        return len(getattr(profile, meth).equilibria)
    figure, ax = binary_figure(xscale=xscale, yscale=yscale)
    ax.heatmap_intensity(n_equilibria,
                         x_left_label=xyy_to_profile.x_left_label,
                         x_right_label=xyy_to_profile.x_right_label,
                         y_left_label=xyy_to_profile.y_left_label,
                         y_right_label=xyy_to_profile.y_right_label,
                         reverse_right=reverse_right,
                         **kwargs)
    ax.annotate_condorcet(left_order=xyy_to_profile.left_order,
                          right_order=xyy_to_profile.right_order,
                          d_order_fixed_share=xyy_to_profile.d_order_fixed_share)
    ax.set_title(title)
    return figure, ax


def binary_plot_winners_at_equilibrium(xyy_to_profile, xscale, yscale, title='Winners at equilibrium',
                                       legend_title='Winners', meth='analyzed_strategies_ordinal',
                                       reverse_right=False, **kwargs):
    """Shortcut: binary plot for the winners at equilibrium.

    Parameters
    ----------
    xyy_to_profile : XyyToProfile
        This is responsible for generating the profiles.
    xscale : Number
        Scale of the plot (resolution) on the x-axis.
    yscale : Number
        Scale of the plot (resolution) on the y-axis.
    title : str
        Title of the plot.
    legend_title : str
        Title of the legend of the plot.
    meth : str
        The name of the :class:`AnalyzedStrategies` property used to count the equilibria. Cf. :class:`Profile`.
    reverse_right : bool
        If True, then the y-axis on the right goes decreasing from 1 to 0 (whereas the y-axis on the left goes
        increasing from 0 to 1).
    kwargs
        Other keyword arguments are passed to the function :meth:`BinaryAxesSubplotPoisson.heatmap_candidates`.

    Examples
    --------
        >>> xyy_to_profile = XyyToProfile(ProfileNoisyDiscrete, left_ranking='bca', right_ranking='cab', noise=0.01)
        >>> figure, ax = binary_plot_winners_at_equilibrium(xyy_to_profile, xscale=5, yscale=5)
    """
    def winners_at_equilibrium(x, y1, y2):
        profile = xyy_to_profile(x, y1, y2)
        return candidates_to_probabilities(getattr(profile, meth).winners_at_equilibrium)
    figure, ax = binary_figure(xscale=xscale, yscale=yscale)
    ax.heatmap_candidates(winners_at_equilibrium,
                          x_left_label=xyy_to_profile.x_left_label,
                          x_right_label=xyy_to_profile.x_right_label,
                          y_left_label=xyy_to_profile.y_left_label,
                          y_right_label=xyy_to_profile.y_right_label,
                          reverse_right=reverse_right,
                          legend_style='color_patches',
                          legend_title=legend_title,
                          **kwargs)
    ax.annotate_condorcet(left_order=xyy_to_profile.left_order,
                          right_order=xyy_to_profile.right_order,
                          d_order_fixed_share=xyy_to_profile.d_order_fixed_share)
    ax.set_title(title)
    return figure, ax


def binary_plot_winning_frequencies(xyy_to_profile, xscale, yscale,
                                    n_max_episodes, init='sincere', samples_per_point=1,
                                    perception_update_ratio=one_over_log_t_plus_one,
                                    ballot_update_ratio=one_over_log_t_plus_one,
                                    winning_frequency_update_ratio=one_over_log_t_plus_one,
                                    title='Winning frequencies', legend_title='Winners',
                                    meth='fictitious_play', reverse_right=False, **kwargs):
    """Shortcut: binary plot for the winning frequencies in fictitious play / iterated voting.

    Parameters
    ----------
    xyy_to_profile : XyyToProfile
        This is responsible for generating the profiles.
    xscale : Number
        Scale of the plot (resolution) on the x-axis.
    yscale : Number
        Scale of the plot (resolution) on the y-axis.
    n_max_episodes : int
        Maximum number of episodes for the fictitious play / iterated voting.
    init : Strategy or TauVector or str
        Cf. :meth:`~poisson_approval.ProfileCardinal.fictitious_play` or :meth:`~poisson_approval.ProfileCardinal.iterated_voting`.
    samples_per_point : int
        How many trials are made for each point drawn. Useful only when initialization is random.
    perception_update_ratio,ballot_update_ratio,winning_frequency_update_ratio : callable or Number
        Cf. :meth:`~poisson_approval.ProfileCardinal.fictitious_play` or
        :meth:`~poisson_approval.ProfileCardinal.iterated_voting`.
    title : str
        Title of the plot.
    legend_title : str
        Title of the legend of the plot.
    meth : str
        The name of the method (``'fictitious_play'`` or ``'iterated_voting'``).
    reverse_right : bool
        If True, then the y-axis on the right goes decreasing from 1 to 0 (whereas the y-axis on the left goes
        increasing from 0 to 1).
    kwargs
        Other keyword arguments are passed to the function :meth:`BinaryAxesSubplotPoisson.heatmap_candidates`.

    Examples
    --------
        >>> xyy_to_profile = XyyToProfile(ProfileNoisyDiscrete, left_ranking='bca', right_ranking='cab', noise=0.01)
        >>> figure, ax = binary_plot_winning_frequencies(xyy_to_profile, xscale=5, yscale=5, n_max_episodes=10)
    """
    def winning_frequencies(x, y1, y2):
        profile = xyy_to_profile(x, y1, y2)
        a_candidate_value = np.zeros(3)
        for _ in range(samples_per_point):
            results = getattr(profile, meth)(init=init, n_max_episodes=n_max_episodes,
                                             perception_update_ratio=perception_update_ratio,
                                             ballot_update_ratio=ballot_update_ratio,
                                             winning_frequency_update_ratio=winning_frequency_update_ratio)
            a_candidate_value = a_candidate_value + d_candidate_value_to_array(results['d_candidate_winning_frequency'])
        return a_candidate_value / samples_per_point

    figure, ax = binary_figure(xscale=xscale, yscale=yscale)
    ax.heatmap_candidates(winning_frequencies,
                          x_left_label=xyy_to_profile.x_left_label,
                          x_right_label=xyy_to_profile.x_right_label,
                          y_left_label=xyy_to_profile.y_left_label,
                          y_right_label=xyy_to_profile.y_right_label,
                          reverse_right=reverse_right,
                          legend_style='palette',
                          legend_title=legend_title,
                          **kwargs)
    ax.annotate_condorcet(left_order=xyy_to_profile.left_order,
                          right_order=xyy_to_profile.right_order,
                          d_order_fixed_share=xyy_to_profile.d_order_fixed_share)
    ax.set_title(title)
    return figure, ax


def binary_plot_convergence(xyy_to_profile, xscale, yscale,
                            n_max_episodes, init='sincere', samples_per_point=1,
                            perception_update_ratio=one_over_log_t_plus_one,
                            ballot_update_ratio=one_over_log_t_plus_one,
                            title='Convergence frequency',
                            meth='fictitious_play', reverse_right=False, **kwargs):
    """Shortcut: binary plot for the convergence frequency in fictitious play / iterated voting.

    Convergence frequency: out of `samples_per_points` trials, in which proportion of the cases did fictitious play or
    iterated voting converge within `n_max_episodes` iterations?

    Parameters
    ----------
    xyy_to_profile : XyyToProfile
        This is responsible for generating the profiles.
    xscale : Number
        Scale of the plot (resolution) on the x-axis.
    yscale : Number
        Scale of the plot (resolution) on the y-axis.
    n_max_episodes : int
        Maximum number of episodes for the fictitious play / iterated voting.
    init : Strategy or TauVector or str
        Cf. :meth:`~poisson_approval.ProfileCardinal.fictitious_play` or :meth:`~poisson_approval.ProfileCardinal.iterated_voting`.
    samples_per_point : int
        How many trials are made for each point drawn. Useful only when initialization is random.
    perception_update_ratio,ballot_update_ratio : callable or Number
        Cf. :meth:`~poisson_approval.ProfileCardinal.fictitious_play` or
        :meth:`~poisson_approval.ProfileCardinal.iterated_voting`.
    title : str
        Title of the plot.
    meth : str
        The name of the method (``'fictitious_play'`` or ``'iterated_voting'``).
    reverse_right : bool
        If True, then the y-axis on the right goes decreasing from 1 to 0 (whereas the y-axis on the left goes
        increasing from 0 to 1).
    kwargs
        Other keyword arguments are passed to the function :meth:`BinaryAxesSubplotPoisson.heatmap_intensity`.

    Examples
    --------
        >>> xyy_to_profile = XyyToProfile(ProfileNoisyDiscrete, left_ranking='bca', right_ranking='cab', noise=0.01)
        >>> figure, ax = binary_plot_convergence(xyy_to_profile, xscale=5, yscale=5, n_max_episodes=10)
    """
    def convergence_frequency(x, y1, y2):
        profile = xyy_to_profile(x, y1, y2)
        n_convergences = 0
        for _ in range(samples_per_point):
            results = getattr(profile, meth)(init=init, n_max_episodes=n_max_episodes,
                                             perception_update_ratio=perception_update_ratio,
                                             ballot_update_ratio=ballot_update_ratio)
            if results['converges']:
                n_convergences += 1
        return n_convergences / samples_per_point

    figure, ax = binary_figure(xscale=xscale, yscale=yscale)
    ax.heatmap_intensity(convergence_frequency,
                         x_left_label=xyy_to_profile.x_left_label,
                         x_right_label=xyy_to_profile.x_right_label,
                         y_left_label=xyy_to_profile.y_left_label,
                         y_right_label=xyy_to_profile.y_right_label,
                         reverse_right=reverse_right,
                         vmin=0., vmax=1.,
                         **kwargs)
    ax.annotate_condorcet(left_order=xyy_to_profile.left_order,
                          right_order=xyy_to_profile.right_order,
                          d_order_fixed_share=xyy_to_profile.d_order_fixed_share)
    ax.set_title(title)
    return figure, ax
